@@ -4,11 +4,13 @@
 
     Tests the C++ Domain
 
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2022 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+import itertools
 import re
+import zlib
 
 import pytest
 
@@ -17,9 +19,9 @@ from sphinx import addnodes
 from sphinx.addnodes import desc
 from sphinx.domains.cpp import (DefinitionError, DefinitionParser, NoOldIdError, Symbol,
                                 _id_prefix, _max_id)
+from sphinx.ext.intersphinx import load_mappings, normalize_intersphinx_mapping
 from sphinx.testing import restructuredtext
 from sphinx.testing.util import assert_node
-from sphinx.util import docutils
 
 
 def parse(name, string):
@@ -72,6 +74,7 @@ def _check(name, input, idDict, output, key, asTextOutput):
         print("Input:    ", input)
         print("astext(): ", resAsText)
         print("Expected: ", outputAsText)
+        print("Node:", parentNode)
         raise DefinitionError("")
 
     idExpected = [None]
@@ -115,15 +118,18 @@ def check(name, input, idDict, output=None, key=None, asTextOutput=None):
            asTextOutput + ';' if asTextOutput is not None else None)
 
 
-def test_fundamental_types():
+def test_domain_cpp_ast_fundamental_types():
     # see https://en.cppreference.com/w/cpp/language/types
     for t, id_v2 in cppDomain._id_fundamental_v2.items():
         def makeIdV1():
             if t == 'decltype(auto)':
                 return None
-            id = t.replace(" ", "-").replace("long", "l").replace("int", "i")
+            id = t.replace(" ", "-").replace("long", "l")
+            if "__int" not in t:
+                id = id.replace("int", "i")
             id = id.replace("bool", "b").replace("char", "c")
             id = id.replace("wc_t", "wchar_t").replace("c16_t", "char16_t")
+            id = id.replace("c8_t", "char8_t")
             id = id.replace("c32_t", "char32_t")
             return "f__%s" % id
 
@@ -132,10 +138,20 @@ def test_fundamental_types():
             if t == "std::nullptr_t":
                 id = "NSt9nullptr_tE"
             return "1f%s" % id
-        check("function", "void f(%s arg)" % t, {1: makeIdV1(), 2: makeIdV2()})
+        id1 = makeIdV1()
+        id2 = makeIdV2()
+        input = "void f(%s arg)" % t.replace(' ', '  ')
+        output = "void f(%s arg)" % t
+        check("function", input, {1: id1, 2: id2}, output=output)
+        if ' ' in t:
+            # try permutations of all components
+            tcs = t.split()
+            for p in itertools.permutations(tcs):
+                input = "void f(%s arg)" % ' '.join(p)
+                check("function", input, {1: id1, 2: id2})
 
 
-def test_expressions():
+def test_domain_cpp_ast_expressions():
     def exprCheck(expr, id, id4=None):
         ids = 'IE1CIA%s_1aE'
         # call .format() on the expr to unescape double curly braces
@@ -171,31 +187,36 @@ def test_expressions():
     exprCheck('nullptr', 'LDnE')
     exprCheck('true', 'L1E')
     exprCheck('false', 'L0E')
-    ints = ['5', '0', '075', '0x0123456789ABCDEF', '0XF', '0b1', '0B1']
+    ints = ['5', '0', '075', '0x0123456789ABCDEF', '0XF', '0b1', '0B1',
+            "0b0'1'0", "00'1'2", "0x0'1'2", "1'2'3"]
     unsignedSuffix = ['', 'u', 'U']
     longSuffix = ['', 'l', 'L', 'll', 'LL']
     for i in ints:
         for u in unsignedSuffix:
             for l in longSuffix:
                 expr = i + u + l
-                exprCheck(expr, 'L' + expr + 'E')
+                exprCheck(expr, 'L' + expr.replace("'", "") + 'E')
                 expr = i + l + u
-                exprCheck(expr, 'L' + expr + 'E')
+                exprCheck(expr, 'L' + expr.replace("'", "") + 'E')
     decimalFloats = ['5e42', '5e+42', '5e-42',
                      '5.', '5.e42', '5.e+42', '5.e-42',
                      '.5', '.5e42', '.5e+42', '.5e-42',
-                     '5.0', '5.0e42', '5.0e+42', '5.0e-42']
+                     '5.0', '5.0e42', '5.0e+42', '5.0e-42',
+                     "1'2'3e7'8'9", "1'2'3.e7'8'9",
+                     ".4'5'6e7'8'9", "1'2'3.4'5'6e7'8'9"]
     hexFloats = ['ApF', 'Ap+F', 'Ap-F',
                  'A.', 'A.pF', 'A.p+F', 'A.p-F',
                  '.A', '.ApF', '.Ap+F', '.Ap-F',
-                 'A.B', 'A.BpF', 'A.Bp+F', 'A.Bp-F']
+                 'A.B', 'A.BpF', 'A.Bp+F', 'A.Bp-F',
+                 "A'B'Cp1'2'3", "A'B'C.p1'2'3",
+                 ".D'E'Fp1'2'3", "A'B'C.D'E'Fp1'2'3"]
     for suffix in ['', 'f', 'F', 'l', 'L']:
         for e in decimalFloats:
             expr = e + suffix
-            exprCheck(expr, 'L' + expr + 'E')
+            exprCheck(expr, 'L' + expr.replace("'", "") + 'E')
         for e in hexFloats:
             expr = "0x" + e + suffix
-            exprCheck(expr, 'L' + expr + 'E')
+            exprCheck(expr, 'L' + expr.replace("'", "") + 'E')
     exprCheck('"abc\\"cba"', 'LA8_KcE')  # string
     exprCheck('this', 'fpT')
     # character literals
@@ -208,13 +229,13 @@ def test_expressions():
             exprCheck("{}'{}'".format(p, c), t + val)
     # user-defined literals
     for i in ints:
-        exprCheck(i + '_udl', 'clL_Zli4_udlEL' + i + 'EE')
-        exprCheck(i + 'uludl', 'clL_Zli5uludlEL' + i + 'EE')
+        exprCheck(i + '_udl', 'clL_Zli4_udlEL' + i.replace("'", "") + 'EE')
+        exprCheck(i + 'uludl', 'clL_Zli5uludlEL' + i.replace("'", "") + 'EE')
     for f in decimalFloats:
-        exprCheck(f + '_udl', 'clL_Zli4_udlEL' + f + 'EE')
-        exprCheck(f + 'fudl', 'clL_Zli4fudlEL' + f + 'EE')
+        exprCheck(f + '_udl', 'clL_Zli4_udlEL' + f.replace("'", "") + 'EE')
+        exprCheck(f + 'fudl', 'clL_Zli4fudlEL' + f.replace("'", "") + 'EE')
     for f in hexFloats:
-        exprCheck('0x' + f + '_udl', 'clL_Zli4_udlEL0x' + f + 'EE')
+        exprCheck('0x' + f + '_udl', 'clL_Zli4_udlEL0x' + f.replace("'", "") + 'EE')
     for p, t in charPrefixAndIds:
         for c, val in chars:
             exprCheck("{}'{}'_udl".format(p, c), 'clL_Zli4_udlE' + t + val + 'E')
@@ -288,13 +309,16 @@ def test_expressions():
     exprCheck('5 == 42', 'eqL5EL42E')
     exprCheck('5 != 42', 'neL5EL42E')
     exprCheck('5 not_eq 42', 'neL5EL42E')
-    # ['<=', '>=', '<', '>']
+    # ['<=', '>=', '<', '>', '<=>']
     exprCheck('5 <= 42', 'leL5EL42E')
     exprCheck('A <= 42', 'le1AL42E')
     exprCheck('5 >= 42', 'geL5EL42E')
     exprCheck('5 < 42', 'ltL5EL42E')
     exprCheck('A < 42', 'lt1AL42E')
     exprCheck('5 > 42', 'gtL5EL42E')
+    exprCheck('A > 42', 'gt1AL42E')
+    exprCheck('5 <=> 42', 'ssL5EL42E')
+    exprCheck('A <=> 42', 'ss1AL42E')
     # ['<<', '>>']
     exprCheck('5 << 42', 'lsL5EL42E')
     exprCheck('A << 42', 'ls1AL42E')
@@ -346,7 +370,7 @@ def test_expressions():
     exprCheck('a(b(c, 1 + d...)..., e(f..., g))', 'cl1aspcl1b1cspplL1E1dEcl1esp1f1gEE')
 
 
-def test_type_definitions():
+def test_domain_cpp_ast_type_definitions():
     check("type", "public bool b", {1: "b", 2: "1b"}, "{key}bool b", key='typedef')
     check("type", "{key}bool A::b", {1: "A::b", 2: "N1A1bE"}, key='typedef')
     check("type", "{key}bool *b", {1: "b", 2: "1b"}, key='typedef')
@@ -391,7 +415,7 @@ def test_type_definitions():
     check('type', '{key}T = Q<A::operator bool>', {2: '1T'}, key='using')
 
 
-def test_concept_definitions():
+def test_domain_cpp_ast_concept_definitions():
     check('concept', 'template<typename Param> {key}A::B::Concept',
           {2: 'I0EN1A1B7ConceptE'})
     check('concept', 'template<typename A, typename B, typename ...C> {key}Foo',
@@ -402,7 +426,7 @@ def test_concept_definitions():
         parse('concept', 'template<typename T> template<typename U> {key}Foo')
 
 
-def test_member_definitions():
+def test_domain_cpp_ast_member_definitions():
     check('member', '  const  std::string  &  name = 42',
           {1: "name__ssCR", 2: "4name"}, output='const std::string &name = 42')
     check('member', '  const  std::string  &  name', {1: "name__ssCR", 2: "4name"},
@@ -430,8 +454,11 @@ def test_member_definitions():
     # check('member', 'int b : (true ? 8 : a) = 42', {1: 'b__i', 2: '1b'})
     check('member', 'int b : 1 || new int{0}', {1: 'b__i', 2: '1b'})
 
+    check('member', 'inline int n', {1: 'n__i', 2: '1n'})
+    check('member', 'constinit int n', {1: 'n__i', 2: '1n'})
 
-def test_function_definitions():
+
+def test_domain_cpp_ast_function_definitions():
     check('function', 'void f(volatile int)', {1: "f__iV", 2: "1fVi"})
     check('function', 'void f(std::size_t)', {1: "f__std::s", 2: "1fNSt6size_tE"})
     check('function', 'operator bool() const', {1: "castto-b-operatorC", 2: "NKcvbEv"})
@@ -560,6 +587,9 @@ def test_function_definitions():
     check("function", "void f(int *volatile const p)", {1: "f__iPVC", 2: "1fPVCi"})
 
     check('function', 'extern int f()', {1: 'f', 2: '1fv'})
+    check('function', 'consteval int f()', {1: 'f', 2: '1fv'})
+
+    check('function', 'explicit(true) void f()', {1: 'f', 2: '1fv'})
 
     check('function', 'decltype(auto) f()', {1: 'f', 2: "1fv"})
 
@@ -615,8 +645,14 @@ def test_function_definitions():
     # from breathe#441
     check('function', 'auto MakeThingy() -> Thingy*', {1: 'MakeThingy', 2: '10MakeThingyv'})
 
+    # from #8960
+    check('function', 'void f(void (*p)(int, double), int i)', {2: '1fPFvidEi'})
 
-def test_operators():
+    # from #9535 comment
+    check('function', 'void f(void (*p)(int) = &foo)', {2: '1fPFviE'})
+
+
+def test_domain_cpp_ast_operators():
     check('function', 'void operator new()', {1: "new-operator", 2: "nwv"})
     check('function', 'void operator new[]()', {1: "new-array-operator", 2: "nav"})
     check('function', 'void operator delete()', {1: "delete-operator", 2: "dlv"})
@@ -660,6 +696,7 @@ def test_operators():
     check('function', 'void operator>()', {1: "gt-operator", 2: "gtv"})
     check('function', 'void operator<=()', {1: "lte-operator", 2: "lev"})
     check('function', 'void operator>=()', {1: "gte-operator", 2: "gev"})
+    check('function', 'void operator<=>()', {1: None, 2: "ssv"})
     check('function', 'void operator!()', {1: "not-operator", 2: "ntv"})
     check('function', 'void operator not()', {2: "ntv"})
     check('function', 'void operator&&()', {1: "sand-operator", 2: "aav"})
@@ -675,14 +712,14 @@ def test_operators():
     check('function', 'void operator[]()', {1: "subscript-operator", 2: "ixv"})
 
 
-def test_nested_name():
+def test_domain_cpp_ast_nested_name():
     check('class', '{key}::A', {1: "A", 2: "1A"})
     check('class', '{key}::A::B', {1: "A::B", 2: "N1A1BE"})
     check('function', 'void f(::A a)', {1: "f__A", 2: "1f1A"})
     check('function', 'void f(::A::B a)', {1: "f__A::B", 2: "1fN1A1BE"})
 
 
-def test_class_definitions():
+def test_domain_cpp_ast_class_definitions():
     check('class', 'public A', {1: "A", 2: "1A"}, output='{key}A')
     check('class', 'private {key}A', {1: "A", 2: "1A"})
     check('class', '{key}A final', {1: 'A', 2: '1A'})
@@ -713,11 +750,11 @@ def test_class_definitions():
           {2: 'I_DpiE1TIJX(Is)EEE', 3: 'I_DpiE1TIJX2IsEEE'})
 
 
-def test_union_definitions():
+def test_domain_cpp_ast_union_definitions():
     check('union', '{key}A', {2: "1A"})
 
 
-def test_enum_definitions():
+def test_domain_cpp_ast_enum_definitions():
     check('enum', '{key}A', {2: "1A"})
     check('enum', '{key}A : std::underlying_type<B>::type', {2: "1A"})
     check('enum', '{key}A : unsigned int', {2: "1A"})
@@ -728,15 +765,18 @@ def test_enum_definitions():
     check('enumerator', '{key}A = std::numeric_limits<unsigned long>::max()', {2: "1A"})
 
 
-def test_anon_definitions():
+def test_domain_cpp_ast_anon_definitions():
     check('class', '@a', {3: "Ut1_a"}, asTextOutput='class [anonymous]')
     check('union', '@a', {3: "Ut1_a"}, asTextOutput='union [anonymous]')
     check('enum', '@a', {3: "Ut1_a"}, asTextOutput='enum [anonymous]')
     check('class', '@1', {3: "Ut1_1"}, asTextOutput='class [anonymous]')
     check('class', '@a::A', {3: "NUt1_a1AE"}, asTextOutput='class [anonymous]::A')
 
+    check('function', 'int f(int @a)', {1: 'f__i', 2: '1fi'},
+          asTextOutput='int f(int [anonymous])')
 
-def test_templates():
+
+def test_domain_cpp_ast_templates():
     check('class', "A<T>", {2: "IE1AI1TE"}, output="template<> {key}A<T>")
     # first just check which objects support templating
     check('class', "template<> {key}A", {2: "IE1A"})
@@ -842,7 +882,16 @@ def test_templates():
     check('type', 'template<C T = int&> {key}A', {2: 'I_1CE1A'}, key='using')
 
 
-def test_requires_clauses():
+def test_domain_cpp_ast_placeholder_types():
+    check('function', 'void f(Sortable auto &v)', {1: 'f__SortableR', 2: '1fR8Sortable'})
+    check('function', 'void f(const Sortable auto &v)', {1: 'f__SortableCR', 2: '1fRK8Sortable'})
+    check('function', 'void f(Sortable decltype(auto) &v)', {1: 'f__SortableR', 2: '1fR8Sortable'})
+    check('function', 'void f(const Sortable decltype(auto) &v)', {1: 'f__SortableCR', 2: '1fRK8Sortable'})
+    check('function', 'void f(Sortable decltype ( auto ) &v)', {1: 'f__SortableR', 2: '1fR8Sortable'},
+          output='void f(Sortable decltype(auto) &v)')
+
+
+def test_domain_cpp_ast_requires_clauses():
     check('function', 'template<typename T> requires A auto f() -> void requires B',
           {4: 'I0EIQaa1A1BE1fvv'})
     check('function', 'template<typename T> requires A || B or C void f()',
@@ -851,7 +900,7 @@ def test_requires_clauses():
           {4: 'I0EIQooaa1A1Baa1C1DE1fvv'})
 
 
-def test_template_args():
+def test_domain_cpp_ast_template_args():
     # from breathe#218
     check('function',
           "template<typename F> "
@@ -866,7 +915,7 @@ def test_template_args():
           key='using')
 
 
-def test_initializers():
+def test_domain_cpp_ast_initializers():
     idsMember = {1: 'v__T', 2: '1v'}
     idsFunction = {1: 'f__T', 2: '1f1T'}
     idsTemplate = {2: 'I_1TE1fv', 4: 'I_1TE1fvv'}
@@ -900,7 +949,7 @@ def test_initializers():
     check('member', 'T v = T{}', idsMember)
 
 
-def test_attributes():
+def test_domain_cpp_ast_attributes():
     # style: C++
     check('member', '[[]] int f', {1: 'f__i', 2: '1f'})
     check('member', '[ [ ] ] int f', {1: 'f__i', 2: '1f'},
@@ -948,7 +997,7 @@ def test_attributes():
     check('function', 'void f() [[attr1]] [[attr2]]', {1: 'f', 2: '1fv'})
 
 
-def test_xref_parsing():
+def test_domain_cpp_ast_xref_parsing():
     def check(target):
         class Config:
             cpp_id_attributes = ["id_attr"]
@@ -981,7 +1030,7 @@ def filter_warnings(warning, file):
 
 
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
-def test_build_domain_cpp_multi_decl_lookup(app, status, warning):
+def test_domain_cpp_build_multi_decl_lookup(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "lookup-key-overload")
     assert len(ws) == 0
@@ -991,7 +1040,7 @@ def test_build_domain_cpp_multi_decl_lookup(app, status, warning):
 
 
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
-def test_build_domain_cpp_warn_template_param_qualified_name(app, status, warning):
+def test_domain_cpp_build_warn_template_param_qualified_name(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "warn-template-param-qualified-name")
     assert len(ws) == 2
@@ -1000,14 +1049,14 @@ def test_build_domain_cpp_warn_template_param_qualified_name(app, status, warnin
 
 
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
-def test_build_domain_cpp_backslash_ok_true(app, status, warning):
+def test_domain_cpp_build_backslash_ok_true(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "backslash")
     assert len(ws) == 0
 
 
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
-def test_build_domain_cpp_semicolon(app, status, warning):
+def test_domain_cpp_build_semicolon(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "semicolon")
     assert len(ws) == 0
@@ -1015,7 +1064,7 @@ def test_build_domain_cpp_semicolon(app, status, warning):
 
 @pytest.mark.sphinx(testroot='domain-cpp',
                     confoverrides={'nitpicky': True, 'strip_signature_backslash': True})
-def test_build_domain_cpp_backslash_ok_false(app, status, warning):
+def test_domain_cpp_build_backslash_ok_false(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "backslash")
     assert len(ws) == 1
@@ -1023,7 +1072,7 @@ def test_build_domain_cpp_backslash_ok_false(app, status, warning):
 
 
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
-def test_build_domain_cpp_anon_dup_decl(app, status, warning):
+def test_domain_cpp_build_anon_dup_decl(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "anon-dup-decl")
     assert len(ws) == 2
@@ -1032,7 +1081,7 @@ def test_build_domain_cpp_anon_dup_decl(app, status, warning):
 
 
 @pytest.mark.sphinx(testroot='domain-cpp')
-def test_build_domain_cpp_misuse_of_roles(app, status, warning):
+def test_domain_cpp_build_misuse_of_roles(app, status, warning):
     app.builder.build_all()
     ws = filter_warnings(warning, "roles-targets-ok")
     assert len(ws) == 0
@@ -1049,8 +1098,8 @@ def test_build_domain_cpp_misuse_of_roles(app, status, warning):
         ('concept', ['concept']),
         ('enum', ['type', 'enum']),
         ('enumerator', ['enumerator']),
-        ('tParam', ['class', 'struct', 'union', 'func', 'member', 'var', 'type', 'concept', 'enum', 'enumerator', 'functionParam']),
         ('functionParam', ['member', 'var']),
+        ('templateParam', ['class', 'struct', 'union', 'member', 'var', 'type']),
     ]
     warn = []
     for targetType, roles in ok:
@@ -1058,6 +1107,9 @@ def test_build_domain_cpp_misuse_of_roles(app, status, warning):
         for r in allRoles:
             if r not in roles:
                 warn.append("WARNING: cpp:{} targets a {} (".format(r, txtTargetType))
+                if targetType == 'templateParam':
+                    warn.append("WARNING: cpp:{} targets a {} (".format(r, txtTargetType))
+                    warn.append("WARNING: cpp:{} targets a {} (".format(r, txtTargetType))
     warn = list(sorted(warn))
     for w in ws:
         assert "targets a" in w
@@ -1076,10 +1128,8 @@ def test_build_domain_cpp_misuse_of_roles(app, status, warning):
     assert len(ws) == len(warn)
 
 
-@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
-                    reason='docutils-0.13 or above is required')
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'add_function_parentheses': True})
-def test_build_domain_cpp_with_add_function_parentheses_is_True(app, status, warning):
+def test_domain_cpp_build_with_add_function_parentheses_is_True(app, status, warning):
     app.builder.build_all()
 
     def check(spec, text, file):
@@ -1119,10 +1169,8 @@ def test_build_domain_cpp_with_add_function_parentheses_is_True(app, status, war
         check(s, t, f)
 
 
-@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
-                    reason='docutils-0.13 or above is required')
 @pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'add_function_parentheses': False})
-def test_build_domain_cpp_with_add_function_parentheses_is_False(app, status, warning):
+def test_domain_cpp_build_with_add_function_parentheses_is_False(app, status, warning):
     app.builder.build_all()
 
     def check(spec, text, file):
@@ -1163,7 +1211,7 @@ def test_build_domain_cpp_with_add_function_parentheses_is_False(app, status, wa
 
 
 @pytest.mark.sphinx(testroot='domain-cpp')
-def test_xref_consistency(app, status, warning):
+def test_domain_cpp_build_xref_consistency(app, status, warning):
     app.builder.build_all()
 
     test = 'xref_consistency.html'
@@ -1199,7 +1247,7 @@ not found in `{test}`
     cpp_any_role = RoleClasses('cpp-any', 'a', ['code'])
     # NYI: consistent looks
     # texpr_role = RoleClasses('cpp-texpr', 'span', ['a', 'code'])
-    expr_role = RoleClasses('cpp-expr', 'code', ['a'])
+    expr_role = RoleClasses('cpp-expr', 'span', ['a'])
     texpr_role = RoleClasses('cpp-texpr', 'span', ['a', 'span'])
 
     # XRefRole-style classes
@@ -1216,8 +1264,7 @@ not found in `{test}`
     for role in (expr_role, texpr_role):
         name = role.name
         expect = '`{name}` puts the domain and role classes at its root'.format(name=name)
-        # NYI: xref should go in the references
-        assert {'xref', 'cpp', name} <= role.classes, expect
+        assert {'sig', 'sig-inline', 'cpp', name} <= role.classes, expect
 
     # reference classes
 
@@ -1227,7 +1274,77 @@ not found in `{test}`
     assert any_role.classes == texpr_role.content_classes['a'], expect
 
 
-def test_noindexentry(app):
+@pytest.mark.sphinx(testroot='domain-cpp', confoverrides={'nitpicky': True})
+def test_domain_cpp_build_field_role(app, status, warning):
+    app.builder.build_all()
+    ws = filter_warnings(warning, "field-role")
+    assert len(ws) == 0
+
+
+@pytest.mark.sphinx(testroot='domain-cpp-intersphinx', confoverrides={'nitpicky': True})
+def test_domain_cpp_build_intersphinx(tempdir, app, status, warning):
+    origSource = """\
+.. cpp:class:: _class
+.. cpp:struct:: _struct
+.. cpp:union:: _union
+.. cpp:function:: void _function()
+.. cpp:member:: int _member
+.. cpp:var:: int _var
+.. cpp:type:: _type
+.. cpp:concept:: template<typename T> _concept
+.. cpp:enum:: _enum
+
+    .. cpp:enumerator:: _enumerator
+
+.. cpp:enum-struct:: _enumStruct
+
+    .. cpp:enumerator:: _scopedEnumerator
+
+.. cpp:enum-class:: _enumClass
+.. cpp:function:: void _functionParam(int param)
+.. cpp:function:: template<typename TParam> void _templateParam()
+"""  # noqa
+    inv_file = tempdir / 'inventory'
+    inv_file.write_bytes(b'''\
+# Sphinx inventory version 2
+# Project: C Intersphinx Test
+# Version: 
+# The remainder of this file is compressed using zlib.
+''' + zlib.compress(b'''\
+_class cpp:class 1 index.html#_CPPv46$ -
+_concept cpp:concept 1 index.html#_CPPv4I0E8$ -
+_concept::T cpp:templateParam 1 index.html#_CPPv4I0E8_concept -
+_enum cpp:enum 1 index.html#_CPPv45$ -
+_enum::_enumerator cpp:enumerator 1 index.html#_CPPv4N5_enum11_enumeratorE -
+_enumClass cpp:enum 1 index.html#_CPPv410$ -
+_enumStruct cpp:enum 1 index.html#_CPPv411$ -
+_enumStruct::_scopedEnumerator cpp:enumerator 1 index.html#_CPPv4N11_enumStruct17_scopedEnumeratorE -
+_enumerator cpp:enumerator 1 index.html#_CPPv4N5_enum11_enumeratorE -
+_function cpp:function 1 index.html#_CPPv49_functionv -
+_functionParam cpp:function 1 index.html#_CPPv414_functionParami -
+_functionParam::param cpp:functionParam 1 index.html#_CPPv414_functionParami -
+_member cpp:member 1 index.html#_CPPv47$ -
+_struct cpp:class 1 index.html#_CPPv47$ -
+_templateParam cpp:function 1 index.html#_CPPv4I0E14_templateParamvv -
+_templateParam::TParam cpp:templateParam 1 index.html#_CPPv4I0E14_templateParamvv -
+_type cpp:type 1 index.html#_CPPv45$ -
+_union cpp:union 1 index.html#_CPPv46$ -
+_var cpp:member 1 index.html#_CPPv44$ -
+'''))  # noqa
+    app.config.intersphinx_mapping = {
+        'https://localhost/intersphinx/cpp/': inv_file,
+    }
+    app.config.intersphinx_cache_limit = 0
+    # load the inventory and check if it's done correctly
+    normalize_intersphinx_mapping(app, app.config)
+    load_mappings(app)
+
+    app.builder.build_all()
+    ws = filter_warnings(warning, "index")
+    assert len(ws) == 0
+
+
+def test_domain_cpp_parse_noindexentry(app):
     text = (".. cpp:function:: void f()\n"
             ".. cpp:function:: void g()\n"
             "   :noindexentry:\n")
@@ -1237,7 +1354,7 @@ def test_noindexentry(app):
     assert_node(doctree[2], addnodes.index, entries=[])
 
 
-def test_mix_decl_duplicate(app, warning):
+def test_domain_cpp_parse_mix_decl_duplicate(app, warning):
     # Issue 8270
     text = (".. cpp:struct:: A\n"
             ".. cpp:function:: void A()\n"
